// Type declarations

// Most people just pass NULL for this structure, so for now
// it's just defined as any to provide maximum flexibility
#define SECURITY_ATTRIBUTES long
#define SECURITY_DESCRIPTOR long

// Some VB appropriate substitutions
#define DWORD long
#define WORD short
#define HANDLE long
#define UINT long
#define BYTE unsigned char
#define HBITMAP long
#define BOOL long
#define HMENU long
#define FLOAT float
#define CHAR unsigned char

// Also exposed publically in kernel section
#define _MAX_PATH 260


#define MAX_LEADBYTES        12             /* 5 ranges, 2 bytes ea., 0 term. */
#define MAX_DEFAULTCHAR      2              /* single or double byte */



typedef [helpstring("Rectangle structure")] struct {
        long Left;
        long Top;
        long Right;
        long Bottom;
} RECT;

typedef [helpstring("Point")] struct {
        long x;
        long y;
} POINTAPI;

typedef [helpstring("Size structure")] struct {
        long cx;
        long cy;
} SIZE;

typedef [helpstring ("64 bit large integer")] struct {
    long dwLowDateTime;
    long dwHighDateTime;
} FILETIME;

typedef [helpstring ("Structure used for asynchronous data transfers")] struct {
    long Internal;
    long InternalHigh;
    long Offset;
    long OffsetHigh;
    long hEvent;
} OVERLAPPED;

typedef struct STARTUPINFOA {
    DWORD   cb;
    long   lpReserved;
    long   lpDesktop;
    long   lpTitle;
    DWORD   dwX;
    DWORD   dwY;
    DWORD   dwXSize;
    DWORD   dwYSize;
    DWORD   dwXCountChars;
    DWORD   dwYCountChars;
    DWORD   dwFillAttribute;
    DWORD   dwFlags;
    WORD    wShowWindow;
    WORD    cbReserved2;
    long    lpReserved2;
    HANDLE  hStdInput;
    HANDLE  hStdOutput;
    HANDLE  hStdError;
} STARTUPINFO;

typedef struct {
    HANDLE hProcess;
    HANDLE hThread;
    DWORD dwProcessId;
    DWORD dwThreadId;
} PROCESS_INFORMATION;

typedef [helpstring("System time structure")] struct {
    WORD wYear;
    WORD wMonth;
    WORD wDayOfWeek;
    WORD wDay;
    WORD wHour;
    WORD wMinute;
    WORD wSecond;
    WORD wMilliseconds;
} SYSTEMTIME;

typedef struct {
    DWORD dwFileAttributes;
    FILETIME ftCreationTime;
    FILETIME ftLastAccessTime;
    FILETIME ftLastWriteTime;
    DWORD nFileSizeHigh;
    DWORD nFileSizeLow;
    DWORD dwReserved0;
    DWORD dwReserved1;
    unsigned char   cFileName[_MAX_PATH];
    unsigned char   cAlternateFileName[ 14 ];
} WIN32_FIND_DATA;

typedef [helpstring("Communications DCB Structure")] struct {
    [helpstring("Length of structure")] long DCBlength;
    [helpstring("Baudrate")] long BaudRate;
    [helpstring("Flag settings")] long Bits1;
    short wReserved;
    [helpstring("Transmit X-ON threshold")] short XonLim;
    [helpstring("Transmit X-OFF threshold")] short XoffLim;
    [helpstring("Number of bits/byte, 4-8")] unsigned char ByteSize;
    [helpstring("0-4=None,Odd,Even,Mark,Space")] unsigned char Parity;
    [helpstring("0,1,2 = 1, 1.5, 2")] unsigned char StopBits;
    [helpstring("Tx and Rx X-ON character")] unsigned char XonChar;
    [helpstring("Tx and Rx X-OFF character")] unsigned char XoffChar;
    [helpstring("Error replacement char")] unsigned char ErrorChar;
    [helpstring("End of Input character")] unsigned char EofChar;
    [helpstring("Received Event character")] unsigned char EvtChar;
    short wReserved1;
} DCB;

typedef [helpstring("Communications Timeouts")] struct {
    [helpstring("Maximum time between read chars")] long ReadIntervalTimeout;
    [helpstring("Multiplier of characters.")] long ReadTotalTimeoutMultiplier;
    [helpstring("Constant in milliseconds")] long ReadTotalTimeoutConstant;
    [helpstring("Constant in milliseconds")] long WriteTotalTimeoutMultiplier;
    [helpstring("Constant in milliseconds")] long WriteTotalTimeoutConstant;
} COMMTIMEOUTS;


typedef [helpstring("COMM status structure")] struct {
    [helpstring("Status flags")] long Bits;
    [helpstring("Bytes in input buffer")] long cbInQue;
    [helpstring("Bytes in output buffer")] long cbOutQue;
} COMSTAT;

typedef [helpstring("COMM configuration structure")] struct {
    [helpstring("Size of the entire struct")] long dwSize;               /* Size of the entire struct */
    [helpstring("version of the structure")] short wVersion;              /* version of the structure */
    short wReserved;             /* alignment */
    [helpstring("device control block")] DCB dcb;                    /* device control block */
    [helpstring("ordinal value for identifying provider-defined data structure format")] long dwProviderSubType;
    [helpstring("Specifies the offset of provider specific data field in bytes from the start")] long dwProviderOffset;
    [helpstring("size of the provider-specific data field")] long dwProviderSize;
    [helpstring("provider-specific data")] short wcProviderData[1];
} COMMCONFIG;

typedef struct {
    WORD wPacketLength;
    WORD wPacketVersion;
    DWORD dwServiceMask;
    DWORD dwReserved1;
    DWORD dwMaxTxQueue;
    DWORD dwMaxRxQueue;
    DWORD dwMaxBaud;
    DWORD dwProvSubType;
    DWORD dwProvCapabilities;
    DWORD dwSettableParams;
    DWORD dwSettableBaud;
    WORD wSettableData;
    WORD wSettableStopParity;
    DWORD dwCurrentTxQueue;
    DWORD dwCurrentRxQueue;
    DWORD dwProvSpec1;
    DWORD dwProvSpec2;
    short wcProvChar;
} COMMPROP;

typedef [helpstring("Code page information structure")] struct {
    [helpstring("max length (bytes) of a char")] long MaxCharSize;
    [helpstring("default character")] unsigned char DefaultChar[MAX_DEFAULTCHAR];
    [helpstring("lead byte ranges")] unsigned char LeadByte[MAX_LEADBYTES];
} CPINFO;

typedef [helpstring("Currency format structure")] struct {
    [helpstring("number of decimal digits")] UINT    NumDigits;
    [helpstring("if leading zero in decimal fields")] UINT LeadingZero;
    [helpstring("group size left of decimal")] UINT    Grouping;
    [helpstring("ptr to decimal separator string")] long   lpDecimalSep;
    [helpstring("ptr to thousand separator string")] long   lpThousandSep;
    [helpstring("negative currency ordering")] UINT    NegativeOrder;
    [helpstring("positive currency ordering")] UINT    PositiveOrder;
    [helpstring("ptr to currency symbol string")] long   lpCurrencySymbol;
} CURRENCYFMT;

typedef [helpstring("File Information Structure")] struct {
    DWORD dwFileAttributes;
    FILETIME ftCreationTime;
    FILETIME ftLastAccessTime;
    FILETIME ftLastWriteTime;
    DWORD dwVolumeSerialNumber;
    DWORD nFileSizeHigh;
    DWORD nFileSizeLow;
    DWORD nNumberOfLinks;
    DWORD nFileIndexHigh;
    DWORD nFileIndexLow;
} BY_HANDLE_FILE_INFORMATION;


typedef [helpstring("Number format structure")] struct {
    [helpstring("number of decimal digits")] UINT    NumDigits;
    [helpstring("if leading zero in decimal fields")] UINT    LeadingZero;
    [helpstring("group size left of decimal")] UINT    Grouping;
    [helpstring("ptr to decimal separator string")] long   lpDecimalSep;
    [helpstring("ptr to thousand separator string")] long   lpThousandSep;
    [helpstring("negative number ordering")] UINT    NegativeOrder;
} NUMBERFMT;

typedef [helpstring("System Informaiton structure")] struct {
    WORD wProcessorArchitecture;
    WORD wReserved;
    DWORD dwPageSize;
    long lpMinimumApplicationAddress;
    long lpMaximumApplicationAddress;
    DWORD dwActiveProcessorMask;
    DWORD dwNumberOfProcessors;
    DWORD dwProcessorType;
    DWORD dwAllocationGranularity;
    WORD wProcessorLevel;
    WORD wProcessorRevision;
} SYSTEM_INFO;

typedef [helpstring("System power status")] struct {
    BYTE ACLineStatus;
    BYTE BatteryFlag;
    BYTE BatteryLifePercent;
    BYTE Reserved1;
    DWORD BatteryLifeTime;
    DWORD BatteryFullLifeTime;
} SYSTEM_POWER_STATUS;

typedef struct {
    LONG Bias;
    short StandardName[ 32 ];
    SYSTEMTIME StandardDate;
    long StandardBias;
    short DaylightName[ 32 ];
    SYSTEMTIME DaylightDate;
    long DaylightBias;
} TIME_ZONE_INFORMATION;

typedef [helpstring("OS version information structure")] struct {
    DWORD dwOSVersionInfoSize;
    DWORD dwMajorVersion;
    DWORD dwMinorVersion;
    DWORD dwBuildNumber;
    DWORD dwPlatformId;
    CHAR   szCSDVersion[ 128 ];
} OSVERSIONINFO;

typedef struct {
    DWORD dwLength;
    DWORD dwMemoryLoad;
    DWORD dwTotalPhys;
    DWORD dwAvailPhys;
    DWORD dwTotalPageFile;
    DWORD dwAvailPageFile;
    DWORD dwTotalVirtual;
    DWORD dwAvailVirtual;
} MEMORYSTATUS;

typedef struct {
    BYTE cBytes;
    BYTE fFixedDisk;
    WORD nErrCode;
    WORD Reserved1;
    WORD Reserved2;
    CHAR szPathName[128];
} OFSTRUCT;

typedef [helpstring("Structure describing a block of memory")] struct {
    long BaseAddress;
    long AllocationBase;
    DWORD AllocationProtect;
    DWORD RegionSize;
    DWORD State;
    DWORD Protect;
    DWORD Type;
} MEMORY_BASIC_INFORMATION;

typedef struct {
    BOOL    fIcon;
    DWORD   xHotspot;
    DWORD   yHotspot;
    HBITMAP hbmMask;
    HBITMAP hbmColor;
} ICONINFO;

typedef struct {
    UINT    cbSize;
    long     iTabLength;
    long     iLeftMargin;
    long     iRightMargin;
    UINT    uiLengthDrawn;
} DRAWTEXTPARAMS;

typedef [helpstring("Window class structure 4.0")] struct {
    UINT        cbSize;
    UINT        style;
    long        lpfnWndProc;
    long        cbClsExtra;
    long        cbWndExtra;
    HANDLE      hInstance;
    HANDLE      hIcon;
    HANDLE      hCursor;
    HANDLE      hbrBackground;
    long        lpszMenuName;
    long        lpszClassName;
    HANDLE      hIconSm;
} WNDCLASSEX;

typedef [helpstring("Window class structure")] struct {
    UINT        cbSize;
    UINT        style;
    long        lpfnWndProc;
    long        cbClsExtra;
    long        cbWndExtra;
    HANDLE      hInstance;
    HANDLE      hIcon;
    HANDLE      hCursor;
    HANDLE      hbrBackground;
    long        lpszMenuName;
    long        lpszClassName;
} WNDCLASS;

typedef [helpstring("Menu item information structure")] struct {
    UINT    cbSize;
    UINT    fMask;
    UINT    fType;
    UINT    fState;
    UINT    wID; 
    HMENU   hSubMenu;
    HBITMAP hbmpChecked;
    HBITMAP hbmpUnchecked;
    DWORD   dwItemData;
    long    dwTypeData;
    UINT    cch;
}   MENUITEMINFO;

typedef struct{
    UINT  length;
    UINT  flags;
    UINT  showCmd;
    POINTAPI ptMinPosition;
    POINTAPI ptMaxPosition;
    RECT  rcNormalPosition;
} WINDOWPLACEMENT;

typedef [helpstring("Track Popup Structure")] struct {
    [helpstring("Size of structure")] UINT cbSize;
    [helpstring("Screen coordinates of rectangle to exclude when positioning")] RECT rcExclude;
} TPMPARAMS;

typedef struct {
    FLOAT   eM11;
    FLOAT   eM12;
    FLOAT   eM21;
    FLOAT   eM22;
    FLOAT   eDx;
    FLOAT   eDy;
} XFORM;

typedef [helpstring("Structure defining a bitmap")] struct {
    LONG        bmType;
    LONG        bmWidth;
    LONG        bmHeight;
    LONG        bmWidthBytes;
    WORD        bmPlanes;
    WORD        bmBitsPixel;
    long        bmBits;
} BITMAP;

typedef [helpstring("Logical brush structure")] struct {
    UINT lbStyle;
    long lbColor;
    LONG lbHatch;
  } LOGBRUSH;

typedef struct {
    BYTE   dmDeviceName[32];
    WORD dmSpecVersion;
    WORD dmDriverVersion;
    WORD dmSize;
    WORD dmDriverExtra;
    DWORD dmFields;
    short dmOrientation;
    short dmPaperSize;
    short dmPaperLength;
    short dmPaperWidth;
    short dmScale;
    short dmCopies;
    short dmDefaultSource;
    short dmPrintQuality;
    short dmColor;
    short dmDuplex;
    short dmYResolution;
    short dmTTOption;
    short dmCollate;
    BYTE   dmFormName[32];
    WORD   dmLogPixels;
    DWORD  dmBitsPerPel;
    DWORD  dmPelsWidth;
    DWORD  dmPelsHeight;
    DWORD  dmDisplayFlags;
    DWORD  dmDisplayFrequency;
    DWORD  dmICMMethod;
    DWORD  dmICMIntent;
    DWORD  dmMediaType;
    DWORD  dmDitherType;
    DWORD  dmICCManufacturer;
    DWORD  dmICCModel;
    DWORD  dmPanningWidth;
    DWORD  dmPanningHeight;
} DEVMODE;

typedef [helpstring("Device independent bitmap structure")] struct {
        DWORD      biSize;
        LONG       biWidth;
        LONG       biHeight;
        WORD       biPlanes;
        WORD       biBitCount;
        DWORD      biCompression;
        DWORD      biSizeImage;
        LONG       biXPelsPerMeter;
        LONG       biYPelsPerMeter;
        DWORD      biClrUsed;
        DWORD      biClrImportant;
} BITMAPINFOHEADER;

typedef [helpstring("Logical font structure")] struct 
{
    LONG      lfHeight;
    LONG      lfWidth;
    LONG      lfEscapement;
    LONG      lfOrientation;
    LONG      lfWeight;
    BYTE      lfItalic;
    BYTE      lfUnderline;
    BYTE      lfStrikeOut;
    BYTE      lfCharSet;
    BYTE      lfOutPrecision;
    BYTE      lfClipPrecision;
    BYTE      lfQuality;
    BYTE      lfPitchAndFamily;
    CHAR      lfFaceName[32];
} LOGFONT;

typedef struct
  {
    UINT        lopnStyle;
    POINTAPI    lopnWidth;
    long        lopnColor;
  } LOGPEN;


typedef struct {
    FLOAT   abcfA;
    FLOAT   abcfB;
    FLOAT   abcfC;
} ABCFLOAT;

typedef struct {
    long     abcA;
    UINT     abcB;
    long     abcC;
} ABC;

typedef struct {
    DWORD   lStructSize;
    long    lpOutString;
    UINT *lpOrder;
    long *lpDx;
    long *lpCaretPos;
    long  lpClass;
    short *lpGlyphs;
    UINT    nGlyphs;
    long     nMaxFit;
} GCP_RESULTS;

typedef [helpstring("Enhanced metafile header")] struct {
    [helpstring("Record type EMR_HEADER")] DWORD   iType;              // 
    [helpstring("Record size in bytes.  This may be greater than the sizeof(ENHMETAHEADER).")] DWORD   nSize;
    [helpstring("Inclusive-inclusive bounds in device units")] RECT   rclBounds;
    [helpstring("Inclusive-inclusive Picture Frame of metafile in .01 mm units")] RECT   rclFrame;
    [helpstring("Signature.  Must be ENHMETA_SIGNATURE.")] DWORD   dSignature;
    [helpstring("Version number")] DWORD   nVersion;
    [helpstring("Size of the metafile in bytes")] DWORD   nBytes;
    [helpstring("Number of records in the metafile")] DWORD   nRecords;
    [helpstring("Number of handles in the handle table")] WORD    nHandles;
    WORD    sReserved;
    [helpstring("Number of chars in the unicode description string")] DWORD   nDescription;
    [helpstring("Offset to the metafile description record.")] DWORD   offDescription;
    [helpstring("Number of entries in the metafile palette.")] DWORD   nPalEntries;
    [helpstring("Size of the reference device in pels")] SIZE   szlDevice;
    [helpstring("Size of the reference device in millimeters")] SIZE   szlMillimeters;
    [helpstring("Size of PIXELFORMATDESCRIPTOR information")] DWORD   cbPixelFormat;
    [helpstring("Offset to PIXELFORMATDESCRIPTOR")] DWORD   offPixelFormat;
    [helpstring("TRUE if OpenGL commands are present in the metafile, otherwise FALSE")] DWORD   bOpenGL;            // 
} ENHMETAHEADER;

typedef struct{
    BYTE        peRed;
    BYTE        peGreen;
    BYTE        peBlue;
    BYTE        peFlags;
} PALETTEENTRY;

typedef struct {
    UINT    gmBlackBoxX;
    UINT    gmBlackBoxY;
    POINTAPI   gmptGlyphOrigin;
    short   gmCellIncX;
    short   gmCellIncY;
} GLYPHMETRICS;

typedef struct {
    WORD    fract;
    short   value;
} FIXED;

typedef struct {
     FIXED  eM11;
     FIXED  eM12;
     FIXED  eM21;
     FIXED  eM22;
} MAT2;

typedef struct tagKERNINGPAIR {
   WORD wFirst;
   WORD wSecond;
   int  iKernAmount;
} KERNINGPAIR;

typedef struct {
    LONG        tmHeight;
    LONG        tmAscent;
    LONG        tmDescent;
    LONG        tmInternalLeading;
    LONG        tmExternalLeading;
    LONG        tmAveCharWidth;
    LONG        tmMaxCharWidth;
    LONG        tmWeight;
    LONG        tmOverhang;
    LONG        tmDigitizedAspectX;
    LONG        tmDigitizedAspectY;
    BYTE        tmFirstChar;
    BYTE        tmLastChar;
    BYTE        tmDefaultChar;
    BYTE        tmBreakChar;
    BYTE        tmItalic;
    BYTE        tmUnderlined;
    BYTE        tmStruckOut;
    BYTE        tmPitchAndFamily;
    BYTE        tmCharSet;
} TEXTMETRIC;

typedef struct {
    LONG        tmHeight;
    LONG        tmAscent;
    LONG        tmDescent;
    LONG        tmInternalLeading;
    LONG        tmExternalLeading;
    LONG        tmAveCharWidth;
    LONG        tmMaxCharWidth;
    LONG        tmWeight;
    LONG        tmOverhang;
    LONG        tmDigitizedAspectX;
    LONG        tmDigitizedAspectY;
    BYTE        tmFirstChar;
    BYTE        tmLastChar;
    BYTE        tmDefaultChar;
    BYTE        tmBreakChar;
    BYTE        tmItalic;
    BYTE        tmUnderlined;
    BYTE        tmStruckOut;
    BYTE        tmPitchAndFamily;
    BYTE        tmCharSet;
    DWORD   ntmFlags;
    UINT    ntmSizeEM;
    UINT    ntmCellHeight;
    UINT    ntmAvgWidth;
} NEWTEXTMETRIC;

typedef struct {
    short   nSize;
    short   wFlags;
    short   nLanguageID;
} RASTERIZER_STATUS;

typedef struct {
    DWORD fsUsb[4];
    DWORD fsCsb[2];
} FONTSIGNATURE;

typedef struct {
    long      x;
    long      y;
    UINT      n;
    long      lpstr;
    UINT      uiFlags;
    RECT      rcl;
    long      *pdx;
} POLYTEXT;

typedef struct {
    LONG        mm;
    LONG        xExt;
    LONG        yExt;
    HANDLE   hMF;
  } METAFILEPICT;

typedef struct {
    int     cbSize;
    long    lpszDocName;
    long    lpszOutput;
    long    lpszDatatype;
    DWORD   fwType;
} DOCINFO;


typedef struct {
    DWORD    dwScope;
    DWORD    dwType;
    DWORD    dwDisplayType;
    DWORD    dwUsage;
    long     lpLocalName;
    long     lpRemoteName;
    long     lpComment ;
    long     lpProvider;
}NETRESOURCE;

typedef struct {
        long pDatatype;
        long pDevMode;
        long DesiredAccess;
} PRINTER_DEFAULTS;

